#include <stdio.h>
#include "boards.h"
#include "app_error.h"
#include "nrf_drv_gpiote.h"
#include "nrf_delay.h"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#include "npmx.h"
#include "npmx_instance.h"
#include "npmx_core.h"
#include <npmx_adc.h>
#include <npmx_charger.h>

#include "drivers/i2c/i2c.h"

#define TWI_INSTANCE_ID         0
#define TWI_IRQ_PRIORITY_LOW    3
#define PM1300_SCL_PIN          (ARDUINO_SCL_PIN)
#define PM1300_SDA_PIN          (ARDUINO_SDA_PIN)
#define NPMX_TWI_ADDRESS        (0x6B)
#define NPMX_INT_PIN NRF_GPIO_PIN_MAP(0, 19)

I2C_DEF(m_i2c_instance, TWI_INSTANCE_ID);

static npmx_instance_t npmx_instance = {0};

/**@brief Function for initializing the nrf log module.
 */
static void log_init(void)
{
    ret_code_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();
}

static void i2c_bus_init(void)
{
    ret_code_t err_code;

    // Firstly need to init i2c instance.
    const nrf_drv_twi_config_t i2c_config = 
    {
       .scl                = PM1300_SCL_PIN,
       .sda                = PM1300_SDA_PIN,
       .frequency          = NRF_DRV_TWI_FREQ_100K,
       .interrupt_priority = APP_IRQ_PRIORITY_HIGH,
       .clear_bus_init     = false
    };

    err_code = i2c_init(&m_i2c_instance, &i2c_config);
    APP_ERROR_CHECK(err_code);
}

__STATIC_INLINE void sys_put_be16(uint16_t val, uint8_t dst[2])
{
	dst[0] = val >> 8;
	dst[1] = val;
}

static npmx_error_t i2c_write_npmx(void *p_context, uint32_t register_address, uint8_t *p_data, size_t num_of_bytes)
{
    ret_code_t err_code;
    uint8_t buffer[2 + num_of_bytes];
    
    sys_put_be16((uint16_t)register_address, buffer);
    memcpy(&buffer[2], p_data, num_of_bytes);

    err_code = i2c_writeBytes(&m_i2c_instance, NPMX_TWI_ADDRESS, buffer, 2 + num_of_bytes, true);
    if (err_code != NRF_SUCCESS) {
        NRF_LOG_INFO("Error writing %d bytes to npmx", num_of_bytes);
        NRF_LOG_FLUSH();
        return NPMX_ERROR_IO;
    }

    return NPMX_SUCCESS;
}

static npmx_error_t i2c_read_npmx(void *p_context, uint32_t register_address, uint8_t *p_data, size_t num_of_bytes)
{
    ret_code_t err_code;
    uint8_t wr_addr[2];
    sys_put_be16((uint16_t)register_address, wr_addr);

    err_code = i2c_writeBytes(&m_i2c_instance, NPMX_TWI_ADDRESS, wr_addr, 2, false);
    if (err_code != NRF_SUCCESS) {
        NRF_LOG_INFO("Error writing register address to npmx");
        NRF_LOG_FLUSH();
        return NPMX_ERROR_IO;
    }

    err_code = i2c_readBytes(&m_i2c_instance, NPMX_TWI_ADDRESS, p_data, num_of_bytes);
    if (err_code != NRF_SUCCESS) {
        NRF_LOG_INFO("Error reading %d bytes from npmx", num_of_bytes);
        NRF_LOG_FLUSH();
        return NPMX_ERROR_IO;
    }

    return NPMX_SUCCESS;
}

/**
 * @brief Callback function to be used when VBUSIN VOLTAGE event occurs.
 *
 * @param[in] p_pm Pointer to the instance of PMIC device.
 * @param[in] type Callback type. Should be @ref NPMX_CALLBACK_TYPE_EVENT_VBUSIN_VOLTAGE.
 * @param[in] mask Received event interrupt mask.
 */
void vbusin_voltage_callback(npmx_instance_t *p_pm, npmx_callback_type_t type, uint8_t mask)
{
    /* Current limit have to be applied each time when USB is (re)connected. */
    if (mask & NPMX_EVENT_GROUP_VBUSIN_DETECTED_MASK) {
        NRF_LOG_INFO("VBUSIN detected");        
        npmx_vbusin_task_trigger(npmx_vbusin_get(p_pm, 0),
                                 NPMX_VBUSIN_TASK_APPLY_CURRENT_LIMIT);
    }
}

static volatile bool npmx_int_trig = false;

__STATIC_INLINE void npmx_interrupt_handler()
{
    npmx_int_trig = true;
}

__STATIC_INLINE void npmx_int_init(void)
{
    ret_code_t err_code;

    nrf_drv_gpiote_in_config_t config = GPIOTE_CONFIG_IN_SENSE_TOGGLE(true);
    config.pull = NRF_GPIO_PIN_PULLDOWN;

    err_code = nrf_drv_gpiote_in_init(NPMX_INT_PIN, &config, (nrfx_gpiote_evt_handler_t)npmx_interrupt_handler);
    APP_ERROR_CHECK(err_code);

    nrf_drv_gpiote_in_event_enable(NPMX_INT_PIN, true);
}


static void npmx_init(void)
{
    if (i2c_ping(&m_i2c_instance, NPMX_TWI_ADDRESS) == NRF_SUCCESS) {
        NRF_LOG_INFO("PM1300 device detected on I2C bus at address 0x%x.", NPMX_TWI_ADDRESS);
    } else {
        NRF_LOG_INFO("PM1300 device not detected on I2C bus at address 0x%x.", NPMX_TWI_ADDRESS);
        return;
    }
    NRF_LOG_FLUSH();

    static npmx_backend_t s_npmx_i2c_backend = {
        .p_write = i2c_write_npmx,
        .p_read = i2c_read_npmx,
        .p_context = NULL,
    };

    npmx_error_t err = NPMX_SUCCESS;
    err = npmx_core_init(&npmx_instance, &s_npmx_i2c_backend, NULL, false);
    if (err != NPMX_SUCCESS) {
        NRF_LOG_INFO("Failed to init NPMX device");
        return;
    }

    /* Get pointer to npmx instance and other required instances. */
    npmx_charger_t *charger_instance = npmx_charger_get(&npmx_instance, 0);
    npmx_vbusin_t *vbusin_instance = npmx_vbusin_get(&npmx_instance, 0);
    npmx_adc_t *adc_instance = npmx_adc_get(&npmx_instance, 0);

    /* Register callback for VBUSIN VOLTAGE event. */
    npmx_core_register_cb(&npmx_instance, vbusin_voltage_callback,
                          NPMX_CALLBACK_TYPE_EVENT_VBUSIN_VOLTAGE);

    /* Enable detecting connected USB. */
    npmx_core_event_interrupt_enable(&npmx_instance, NPMX_EVENT_GROUP_VBUSIN_VOLTAGE,
                                     NPMX_EVENT_GROUP_VBUSIN_DETECTED_MASK);

    /* Disable charger before changing charge current and termination voltage. */
    npmx_charger_module_disable_set(charger_instance, NPMX_CHARGER_MODULE_CHARGER_MASK);

    /* Set charging current. */
    npmx_charger_charging_current_set(charger_instance, CONFIG_CHARGING_CURRENT);

    /* Set maximum discharging current. */
    npmx_charger_discharging_current_set(charger_instance, CONFIG_DISCHARGING_CURRENT);

    /* Set battery termination voltage in normal temperature. */
    npmx_charger_termination_normal_voltage_set(
            charger_instance, npmx_charger_voltage_convert(CONFIG_TERMINATION_VOLTAGE_NORMAL));

    /* Set battery termination voltage in warm temperature. */
    npmx_charger_termination_warm_voltage_set(
            charger_instance, npmx_charger_voltage_convert(CONFIG_TERMINATION_VOLTAGE_WARM));

    /* Enable charger. */
    npmx_charger_module_enable_set(charger_instance, NPMX_CHARGER_MODULE_CHARGER_MASK);

    /* Set the current limit for the USB port.
     * Current limit needs to be applied after each USB (re)connection.
     */
    npmx_vbusin_current_limit_set(vbusin_instance,
                                  npmx_vbusin_current_convert(CONFIG_CURRENT_LIMIT));

    /* Apply current limit. */
    npmx_vbusin_task_trigger(vbusin_instance, NPMX_VBUSIN_TASK_APPLY_CURRENT_LIMIT);

    npmx_adc_ntc_config_t ntc_config = { .type = npmx_adc_ntc_type_convert(
                                                 CONFIG_THERMISTOR_RESISTANCE),
                                         .beta = CONFIG_THERMISTOR_BETA };

    /* Set thermistor type and NTC beta value for ADC measurements. */
    npmx_adc_ntc_config_set(adc_instance, &ntc_config);

    /* Enable auto measurement of the battery current after the battery voltage measurement. */
    npmx_adc_ibat_meas_enable_set(adc_instance, true);

    /* Trigger required ADC measurements. */
    npmx_adc_task_trigger(adc_instance, NPMX_ADC_TASK_SINGLE_SHOT_VBAT);
    npmx_adc_task_trigger(adc_instance, NPMX_ADC_TASK_SINGLE_SHOT_NTC);

    /* Use nPM GPIO as interrupt output. */
    npmx_gpio_mode_set(npmx_gpio_get(&npmx_instance, 0),
                       NPMX_GPIO_MODE_OUTPUT_IRQ);

    if(!nrf_drv_gpiote_is_init()) {
        APP_ERROR_CHECK(nrf_drv_gpiote_init());
    }

}

int read_sensors(npmx_instance_t *const p_pm, float *voltage, float *current, float *temp)
{
    npmx_adc_t *adc_instance = npmx_adc_get(p_pm, 0);
    npmx_adc_meas_all_t meas = {};

    if (npmx_adc_meas_all_get(adc_instance, &meas) != NPMX_SUCCESS) {
            NRF_LOG_INFO("Reading ADC measurements failed.");
            return NPMX_ERROR_IO;
    }

    if (npmx_adc_task_trigger(adc_instance, NPMX_ADC_TASK_SINGLE_SHOT_VBAT) != NPMX_SUCCESS) {
            NRF_LOG_INFO("Triggering VBAT measurement failed.");
            return NPMX_ERROR_IO;
    }

    if (npmx_adc_task_trigger(adc_instance, NPMX_ADC_TASK_SINGLE_SHOT_NTC) != NPMX_SUCCESS) {
            NRF_LOG_INFO("Triggering NTC measurement failed.");
            return NPMX_ERROR_IO;
    }

    /* Convert current in milliamperes to current in amperes. */
    *current = (float)meas.values[NPMX_ADC_MEAS_VBAT2_IBAT] / 1000.0f;

    /* Convert temperature in millidegrees Celsius to temperature in Celsius */
    *temp = (float)meas.values[NPMX_ADC_MEAS_BAT_TEMP] / 1000.0f;

    /* Convert voltage in millivolts to voltage in volts. */
    *voltage = (float)meas.values[NPMX_ADC_MEAS_VBAT] / 1000.0f;

    return 0;
}

/**@brief Application main function.
 */
int main(void)
{
    // Initialize.
    log_init();
    i2c_bus_init();
    npmx_init();
    npmx_int_init();

    // Start execution.
    float voltage;
    float current;
    float temp;
    int ret;
    char tempString[128] = {0};

    // Enter main loop.
    for (;;)
    {
        if (npmx_int_trig == true) {
            npmx_int_trig = false;
            npmx_core_interrupt(&npmx_instance);
            npmx_core_proc(&npmx_instance);
        }

	ret = read_sensors(&npmx_instance, &voltage, &current, &temp);
	if (ret < 0) {
		NRF_LOG_INFO("Error: Could not read data from charger device.");
		return ret;
	}

        sprintf(tempString, "V: %0.2f; I: %0.3f; T: %0.2f", voltage, current, temp);
        NRF_LOG_INFO("%s", tempString);
        NRF_LOG_FLUSH();

        nrf_delay_ms(1000);
    }

}
